/*
 * Panel application "LIFE"
 */
#include <application.h> 

/*
 * function prototypes
 */
int life_init(void);
const char life_name_P[] PROGMEM="Life";
const char life_icon_P[] PROGMEM=
{
  18,18,
  2,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x00,0x00,

  0x00,0x0c,0x00,
  0x00,0x08,0x00,
  0x08,0x00,0x00,
  0x04,0x00,0x00,
  0x06,0x01,0x00,
  0x00,0x03,0x00,
  0x30,0xc3,0x00,
  0x30,0x62,0x00,
  0x00,0x60,0x00,
  0x00,0x20,0x00,
  0x00,0x00,0x00,
  0x01,0x00,0x00,
  0x41,0x41,0x00,
  0xa1,0x83,0x80,
  0xa0,0x01,0x00,
  0x40,0x00,0x00,
  0x00,0x00,0x00,
  0x00,0x30,0x00,
};


struct application LIFE={life_name_P, life_icon_P, life_init, 0, 0};

const char life_edit_text_P[] PROGMEM=
{ 
  0x00, 0xb6, 0x9e, 0x1e, 0x00, 0x00,
  0x7a, 0x9e, 0x1c, 0x9e, 0x1e, 0x9e,
  0x9c, 0x1c, 0x9e, 0xee, 0x0a, 0x00
};
  
  
/*
 * draw a new Life Generation
 */
#define MAXSTAGNATION 3  /* number of equal generations when to skip */ 

#define CRC16INIT	0x0000
#define CRC16POLY 0x1021  //0x1021 = X^16+X^12+X^5+X^0

/*
 * calculate CRC16 checksum for detecting cycles
 */
unsigned int crc16(unsigned char *data_in, unsigned short	number_of_bytes_to_read)
{
	unsigned int   crc;
  unsigned short loop_count;
  unsigned char  bit_counter;

  crc = CRC16INIT;

  for (loop_count = 0; loop_count != number_of_bytes_to_read; loop_count++)
  {
    crc = (data_in[loop_count] << 8) ^ crc;
    for (bit_counter = 0; bit_counter < 8; bit_counter++)
   	{
      //check if MSB is one
     	if (crc & 0x8000)
        crc = (crc << 1) ^ CRC16POLY;
      else
       	crc = crc << 1;
    }
  }
  return crc;
}


int LEDLife(int x, int y, int w, int h, u_char c)
{
	static int generation=0;
//  static int old_population=0; /* number of living cells last generation */
//  static int old_death=0;      /* number of deaths last generation */
//  static int old_birth=0;      /* number of births last generation */
  static int stagnation=0;     /* stagnation count */
  static unsigned int chksum[5];

  int xx,yy;
  int xl,xr,yu,yl;
  int population=0;     /* number of living cells */
  int death=0;          /* number of deaths */
  int birth=0;          /* number of births */
  u_char neighbors;
  u_char cyclic;
  
  if (c&0x80)
  {
    memset(chksum,0,sizeof(chksum));
  	generation=0;
  	stagnation=0;
  	return 1;
  }
  generation++;
  if (generation>10005) return 0;
    
  cyclic=(c&0x40);
  c&=0x03;
      
  if (!w) w=MAXX;
  if (!h) h=MAXY;
  
  /* clear area */
  LEDFRectangle(x,y,w,h,0);

  for (yy=y;yy<y+h;yy++)
  {
		if (yy>y)
	    yu=yy-1;
		else
		  yu=(cyclic)?y+h-1:-1;
		  
		if (yy<y+h-1)
		  yl=yy+1;
		else
  	  yl=(cyclic)?y:-1;
  		  
  	for (xx=x;xx<x+w;xx++)
  	{
  		/* count the neighbors */
  		neighbors=0;

  		if (xx>x) 
		    xl=xx-1;
 		  else
 		    xl=(cyclic)?x+w-1:-1;  
 		    
  		if (xx<x+w-1)
  		  xr=xx+1;
  		else  
  		  xr=(cyclic)?x:-1;
  		  
  		if (yu>=0)
  		{      
  		  if ((xl>=0) && (LEDGetScreenPixel(xl,yu)==c)) neighbors++;   
  	  	if (           (LEDGetScreenPixel(xx,yu)==c)) neighbors++;   
  		  if ((xr>=0) && (LEDGetScreenPixel(xr,yu)==c)) neighbors++;
  	  }
  	  
  		if ((xl>=0)   && (LEDGetScreenPixel(xl,yy)==c)) neighbors++;    
  		if ((xr>=0)   && (LEDGetScreenPixel(xr,yy)==c)) neighbors++;    

      if (yl>=0)
      {
  		  if ((xl>=0) && (LEDGetScreenPixel(xl,yl)==c)) neighbors++;   
  		  if (           (LEDGetScreenPixel(xx,yl)==c)) neighbors++;   
  		  if ((xr>=0) && (LEDGetScreenPixel(xr,yl)==c)) neighbors++;
      }

  		/* is current cell alife */
  		if (LEDGetScreenPixel(xx,yy)==c)
  		{
  			if ((neighbors==2) || (neighbors==3))
  			{
  			  LEDPixel(xx,yy,c);
  			  population++;
  			}
  			else
  			  death++;
  		}
  		else
  			if (neighbors==3)
  			{
  			  LEDPixel(xx,yy,c);  
  			  birth++;
  			  population++;
  			}
  	}
  }
/*
  if ((population==old_population)&&(death==old_death)&&(birth==old_birth))
  {
  	//stagnation++;
  	if (stagnation>MAXSTAGNATION)
  	{
  		old_population=0;
  		old_death=0;
  		old_birth=0;
  		return(0);
  	}
  }
  else
  {
  	old_population=population;
  	old_death=death;
  	old_birth=birth;
  	stagnation=0;
  }	
*/  
  // calculate checksum
  chksum[4]=chksum[3];
  chksum[3]=chksum[2];
  chksum[2]=chksum[1];
  chksum[1]=chksum[0];
  chksum[0]=crc16(LEDGetbuffer(),BUFSIZE);
  if ((chksum[0]==chksum[1])||(chksum[0]==chksum[2])||(chksum[0]==chksum[3])||(chksum[0]==chksum[4])) 
  {
    stagnation++;
    if (stagnation>MAXSTAGNATION) return 0;
  }
  
  printf_P(PSTR("Gen: %3i  Pop: %3i  Birth: %3i  Death: %3i  Chksum: 0x%04X"), generation, population, birth, death,chksum[0]);
  if (stagnation) printf_P(PSTR("  Stag: %3i"), stagnation);
  printf("\n");

  LEDNumout(generation,0);
  LEDNumout(population,4);
  LEDNumout(birth,8);
  LEDNumout(death,12);

  return(generation && (death || birth));
}

void life_edit(int x0, int y0, int w, int h, char color)
{

	int cx,cy;
	char pcolor,ccolor,mode;
	mode=0;
	ccolor=color+1; if (ccolor>YELLOW) ccolor=RED;
	cx=x0+(w>>1);
	cy=y0+(h>>1);
  pcolor=LEDGetPixel(cx,cy);
  LEDPixel(cx,cy,ccolor);
  LEDDraw();
  memset((char *)buf7seg,0,sizeof(buf7seg));
	memcpy_P((char *)buf7seg+5,life_edit_text_P+mode*6,6);
	do
	{
	  if (keypad&BTN_RIGHT)
	  {
	  	if (cx<x0+w-1)
	  	{
	  		LEDPixel(cx,cy,pcolor);
	  		cx++;
        pcolor=LEDGetPixel(cx,cy);
        LEDPixel(cx,cy,ccolor);
        LEDDraw();
	  	}	
	  }
	  if (keypad&BTN_LEFT)
	  {
	  	if (cx>x0)
	  	{
	  		LEDPixel(cx,cy,pcolor);
	  		cx--;
        pcolor=LEDGetPixel(cx,cy);
        LEDPixel(cx,cy,ccolor);
        LEDDraw();
	  	}	
	  }
	  if (keypad&BTN_DOWN)
	  {
	  	if (cy<y0+h-1)
	  	{
	  		LEDPixel(cx,cy,pcolor);
	  		cy++;
        pcolor=LEDGetPixel(cx,cy);
        LEDPixel(cx,cy,ccolor);
        LEDDraw();
	  	}	
	  }
	  if (keypad&BTN_UP)
	  {
	  	if (cy>y0)
	  	{
	  		LEDPixel(cx,cy,pcolor);
	  		cy--;
        pcolor=LEDGetPixel(cx,cy);
        LEDPixel(cx,cy,ccolor);
        LEDDraw();
	  	}	
	  }
	  if (keypad&BTN_ACTRIGHT)
	  {
	  	mode++;	if (mode>2) mode=0;
  		memcpy_P((char *)buf7seg+5,life_edit_text_P+mode*6,6);
  		while(keypad);
	  }
	  if (keypad&BTN_ACTION)
	  {
	  	switch (mode)
	  	{
	  		case 0 : pcolor=color; break;
	  	  case 1 : pcolor=BLACK; break;
	  	  case 2 : pcolor=BLACK; 
	  	           LEDClear(); 
	  	           LEDLife(0,0,0,0,0x80);
	  	           LEDPixel(cx,cy,ccolor); 
	  	           LEDDraw(); 
	  	           mode=0; 
	  	           while (keypad);
  		           memcpy_P((char *)buf7seg+5,life_edit_text_P+mode*6,6);
	  	           break;
	  	}
	  }
	} while ((wait(100)!=3) && (!(keypad&BTN_ACTLEFT)));
	LEDPixel(cx,cy,pcolor);
	while (keypad); 	
}

int life_init(void)
{
	int x,y;
	int lifex, lifey, lifew, lifeh, t;
	char color=RED, newcolor=0;
	
	puts_P(PSTR("Press Ctrl-C to quit"));
  lifex=0; lifey=0; lifew=MAXX; lifeh=MAXY;
  LEDRandomfill(lifex,lifey,lifew,lifeh,color,(lifew*lifeh)/8);
  LEDLife(lifex,lifey,lifew,lifeh,0xA0+color);
  LEDDraw();
  do
  {
  	t=0;
  	do
  	{
      if (wait(10)==3) return 0;
      t++;
    }
    while ((t<70) && !keypad);  
    
    /* Button Action : Kill them all */
    if ((keypad&BTN_ACTION)||(!LEDLife(lifex,lifey,lifew,lifeh,0x40+color)))
    {
    	LEDLine(lifex,lifey,lifex+lifew-1,lifey+lifeh-1,YELLOW);
    	LEDLine(lifex,lifey+lifeh-1,lifex+lifew-1,lifey,YELLOW);
    	LEDDraw();
    	if (wait(1000)==3) return 0;
      LEDFRectangle(lifex,lifey,lifew,lifeh,BLACK);
      LEDRandomfill(lifex,lifey,lifew,lifeh,color,(lifew*lifeh)/10);
      LEDLife(lifex,lifey,lifew,lifeh,0xA0+color);
    }
    
    /* Button Left : quit */
    if (keypad&(BTN_LEFT+BTN_ACTLEFT)) return 0;
    
    /* Button Down : change color */
    if (keypad&(BTN_DOWN+BTN_UP))
    {
    	if (keypad&BTN_DOWN)
    	{
    	  newcolor=color+1; if (newcolor>YELLOW) newcolor=RED;
    	}
    	if (keypad&BTN_UP)
    	{
    	  newcolor=color-1; if (newcolor<RED) newcolor=YELLOW;
    	}
    	
    	for (y=lifey; y<lifey+lifeh; y++)
    	  for (x=lifex; x<lifex+lifew; x++)
    	    if (LEDGetPixel(x,y)==color) LEDPixel(x,y,newcolor);
    	color=newcolor;
    	LEDDraw();
    	while (keypad) ;
    }
    
    /* Button Right : call editor */
    if (keypad&BTN_RIGHT)
      life_edit(lifex,lifey,lifew,lifeh,color);
    LEDDraw();
  } while(1);
  return 0;
}
